@@ -8,3 +8,7 @@
 #undef VERSION
 #undef HAVE_LIBSM
 #undef GETTEXT_PACKAGE
+#undef WE_ARE_LIBGNOMEPRINT_INTERNALS
+#undef GPA_MODULES_DIR
+#undef GNOME_PRINT_MODULES_DIR
+#undef GNOME_PRINT_DATA_DIR
