@@ -23,11 +23,12 @@
  *
  */
 
-#define __GNOME_PRINT_CONFIG_C__
+#include <config.h>
 
 #include <ctype.h>
 #include <stdlib.h>
 #include <locale.h>
+
 #include "gpa/gpa-node-private.h"
 #include "gnome-print-config.h"
 
@@ -36,6 +37,18 @@ struct _GnomePrintConfig {
 	GPANode *node;
 };
 
+GType
+gnome_print_config_get_type (void)
+{
+	static GType type;
+	if (!type)
+		type = g_boxed_type_register_static (
+			"GnomePrintConfig",
+			(GBoxedCopyFunc) gnome_print_config_ref,
+			(GBoxedFreeFunc) gnome_print_config_unref);
+	return type;
+}
+
 GPANode *
 gnome_print_config_get_node (GnomePrintConfig *config)
 {
@@ -327,5 +340,5 @@ gnome_print_config_dump (GnomePrintConfig *config)
 {
 	g_return_if_fail (config != NULL);
 
-	gpa_utils_dump_tree (config->node);
+	gpa_utils_dump_tree (config->node, 2);
 }
