@@ -28,24 +28,32 @@
 
 #define __GNOME_PRINT_C__
 
+#include "config.h"
 #include <string.h>
 #include <gmodule.h>
+
+#ifdef GNOME_PRINT_ENABLE_PAGE_CALLBACKS
 #include <bonobo/bonobo-types.h>
+#endif
 
 #include "gnome-print-i18n.h"
 #include "gnome-print-private.h"
 #include "gp-gc-private.h"
 #include "gnome-print-transport.h"
-#include "gnome-print-ps2.h"
-#include "gnome-print-frgba.h"
 #include "libgnomeprint-marshal.h"
 
+#include "gnome-print-ps2.h"
+#include "gnome-print-pdf.h"
+#include "gnome-print-frgba.h"
+
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
 static void gnome_print_context_init (GnomePrintContext *pc);
 
 static void gnome_print_context_finalize (GObject *object);
 
+#ifdef GNOME_PRINT_ENABLE_MODULE_DRIVERS
 static GnomePrintContext *gnome_print_context_create (gpointer get_type, GnomePrintConfig *config);
+#endif
 
 static GObjectClass *parent_class = NULL;
 
@@ -88,6 +96,7 @@ gnome_print_context_init (GnomePrintContext *pc)
 
 	pc->gc = gp_gc_new ();
 	pc->haspage = FALSE;
+	pc->pages = 0;
 }
 
 static void
@@ -148,7 +157,7 @@ gnome_print_context_create_transport (GnomePrintContext *pc)
 /**
  * gnome_print_beginpage:
  * @pc: A #GnomePrintContext
- * @name: Name of the page
+ * @name: Name of the page, NULL if you just want to use the page number of the page
  *
  * Starts new output page with @name. Naming is used for interactive
  * contexts like #GnomePrintPreview and Document Structuring Convention
@@ -164,16 +173,29 @@ gnome_print_context_create_transport (GnomePrintContext *pc)
 int
 gnome_print_beginpage (GnomePrintContext *pc, const guchar *name)
 {
+	guchar *real_name;
+	
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (!pc->haspage, GNOME_PRINT_ERROR_NOMATCH);
 
+	pc->pages++;
+	if (name == NULL) {
+		real_name = g_strdup_printf ("%d", pc->pages);
+	} else {
+		real_name = (guchar *) name;
+	}
+	
 	gp_gc_reset (pc->gc);
 	pc->haspage = TRUE;
 
 	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->beginpage)
-		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->beginpage (pc, name);
+		return GNOME_PRINT_CONTEXT_GET_CLASS (pc)->beginpage (pc, real_name);
+
+	if (name == NULL) {
+		g_free (real_name);
+	}
 
 	return GNOME_PRINT_OK;
 }
@@ -382,6 +404,8 @@ gnome_print_glyphlist_transform (GnomePrintContext *pc, const gdouble *affine, G
 	return GNOME_PRINT_OK;
 }
 
+
+#ifdef GNOME_PRINT_ENABLE_PAGE_CALLBACKS
 gint
 gnome_print_callback_closure_invoke (GnomePrintContext *pc,
 				     gpointer           pagedata,
@@ -453,6 +477,7 @@ gnome_print_page_callback (GnomePrintContext *pc, GnomePrintPageCallback callbac
 
 	return ret;
 }
+#endif /* GNOME_PRINT_ENABLE_PAGE_CALLBACKS */
 
 /**
  * gnome_print_context_close:
@@ -505,58 +530,75 @@ gnome_print_context_close (GnomePrintContext *pc)
 GnomePrintContext *
 gnome_print_context_new (GnomePrintConfig *config)
 {
-	GnomePrintContext *pc;
+	GnomePrintContext *pc = NULL;
 	guchar *drivername;
-
+#ifdef GNOME_PRINT_ENABLE_MODULE_DRIVERS
+	guchar *modulename;
+#endif
 	g_return_val_if_fail (config != NULL, NULL);
 
 	drivername = gnome_print_config_get (config, "Settings.Engine.Backend.Driver");
-	g_return_val_if_fail (drivername != NULL, NULL);
+	if (drivername == NULL) {
+		drivername = g_strdup ("gnome-print-ps");
+	}
 
-	pc = NULL;
-
-	if (!strcmp (drivername, "gnome-print-ps")) {
+	if (strcmp (drivername, "gnome-print-ps") == 0) {
 		GnomePrintContext *ps;
+		g_print ("Printing to Postscript...\n");
 		ps = gnome_print_ps2_new (config);
 		pc = gnome_print_frgba_new (ps);
 		g_object_unref (G_OBJECT (ps));
-	} else {
-		guchar *modulename;
-		modulename = gnome_print_config_get (config, "Settings.Engine.Backend.Driver.Module");
-		if (modulename) {
-			static GHashTable *modules = NULL;
-			GModule *module;
-			if (!modules) modules = g_hash_table_new (g_str_hash, g_str_equal);
-			module = g_hash_table_lookup (modules, modulename);
-			if (!module) {
-				gchar *path;
-				path = g_module_build_path (GNOME_PRINT_LIBDIR "/drivers", modulename);
-				module = g_module_open (path, G_MODULE_BIND_LAZY);
-				if (module) g_hash_table_insert (modules, g_strdup (modulename), module);
-				g_free (path);
-			}
-			if (module) {
-				gpointer get_type;
-				if (g_module_symbol (module, "gnome_print__driver_get_type", &get_type)) {
-					pc = gnome_print_context_create (get_type, config);
-				} else {
-					g_warning ("Missing gnome_print__driver_get_type in %s\n", modulename);
-					g_module_close (module);
-				}
-			} else {
-				g_warning ("Cannot open module: %s\n", modulename);
-			}
-			g_free (modulename);
-		} else {
-			g_warning ("Unknown driver: %s", drivername);
-		}
+		g_free (drivername);
+		return pc;
+	}
+	
+	if (strcmp (drivername, "gnome-print-pdf") == 0) {
+		g_print ("Printing to PDF ...\n");
+		pc = gnome_print_pdf_new (config);
+		g_free (drivername);
+		return pc;
 	}
 
+#ifdef GNOME_PRINT_ENABLE_MODULE_DRIVERS
+        /* FIXME: This has not been tested yet (Chema) */
+	modulename = gnome_print_config_get (config, "Settings.Engine.Backend.Driver.Module");
+	if (modulename) {
+		static GHashTable *modules = NULL;
+		GModule *module;
+		if (!modules)
+			modules = g_hash_table_new (g_str_hash, g_str_equal);
+		module = g_hash_table_lookup (modules, modulename);
+		if (!module) {
+			gchar *path;
+			path = g_module_build_path (GNOME_PRINT_MODULES_DIR "/drivers", modulename);
+			module = g_module_open (path, G_MODULE_BIND_LAZY);
+			if (module)
+				g_hash_table_insert (modules, g_strdup (modulename), module);
+			g_free (path);
+		}
+		if (module) {
+			gpointer get_type;
+			if (g_module_symbol (module, "gnome_print__driver_get_type", &get_type)) {
+				pc = gnome_print_context_create (get_type, config);
+			} else {
+				g_warning ("Missing gnome_print__driver_get_type in %s\n", modulename);
+				g_module_close (module);
+			}
+		} else {
+			g_warning ("Cannot open module: %s\n", modulename);
+		}
+		g_free (modulename);
+	} else {
+		g_warning ("Unknown driver: %s", drivername);
+	}
+
+#endif
 	g_free (drivername);
 
 	return pc;
 }
 
+#ifdef GNOME_PRINT_ENABLE_MODULE_DRIVERS
 static GnomePrintContext *
 gnome_print_context_create (gpointer get_type, GnomePrintConfig *config)
 {
@@ -574,4 +616,4 @@ gnome_print_context_create (gpointer get_type, GnomePrintConfig *config)
 
 	return pc;
 }
-
+#endif
