@@ -1,30 +1,31 @@
-#define __GPA_NODE_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * This file is part of libgnomeprint 2
+ *  gpa-node.c:
  *
- * Libgnomeprint is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
- * Libgnomeprint is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with the libgnomeprint; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Authors :
- *   Lauris Kaplinski <lauris@ximian.com>
+ *  Authors :
+ *    Lauris Kaplinski <lauris@ximian.com>
+ *    Jose M. Celorio <chema@ximian.com>
  *
- * Copyright (C) 2001 Ximian, Inc.
+ *  Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
  *
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -36,17 +37,13 @@
 /* GPANode */
 
 enum {MODIFIED, LAST_SIGNAL};
+static GObjectClass *parent_class;
+static guint node_signals [LAST_SIGNAL] = {0};
 
 static void gpa_node_class_init (GPANodeClass *klass);
 static void gpa_node_init (GPANode *node);
-
 static void gpa_node_finalize (GObject *object);
 
-static gint gpa_node_modified_idle_hook (GPANode *node);
-
-static GObjectClass *parent_class;
-static guint node_signals[LAST_SIGNAL] = {0};
-
 GType
 gpa_node_get_type (void)
 {
@@ -90,7 +87,7 @@ static void
 gpa_node_init (GPANode *node)
 {
 	node->parent = NULL;
-	node->next = NULL;
+	node->next   = NULL;
 }
 
 static void
@@ -102,7 +99,7 @@ gpa_node_finalize (GObject *object)
 	node = (GPANode *) object;
 
 	g_assert (node->parent == NULL);
-	g_assert (node->next == NULL);
+	g_assert (node->next   == NULL);
 
 	id = GPOINTER_TO_INT (g_object_get_data (object, "idle_id"));
 	if (id != 0) {
@@ -113,8 +110,6 @@ gpa_node_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-/* Methods */
-
 GPANode *
 gpa_node_new (GType type, const guchar *id)
 {
@@ -124,19 +119,8 @@ gpa_node_new (GType type, const guchar *id)
 
 	node = g_object_new (type, NULL);
 
-	if (id) node->qid = gpa_quark_from_string (id);
-
-	return node;
-}
-
-GPANode *
-gpa_node_construct (GPANode *node, const guchar *id)
-{
-	g_return_val_if_fail (node != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
-	g_return_val_if_fail (!node->qid, NULL);
-
-	if (id) node->qid = gpa_quark_from_string (id);
+	if (id)
+		node->qid = gpa_quark_from_string (id);
 
 	return node;
 }
@@ -149,6 +133,9 @@ gpa_node_duplicate (GPANode *node)
 
 	if (GPA_NODE_GET_CLASS (node)->duplicate)
 		return GPA_NODE_GET_CLASS (node)->duplicate (node);
+	else
+		g_warning ("Can't duplicate the \"%s\" node because the \"%s\" Class does not have a "
+			   "duplicate method.", gpa_node_id (node), G_OBJECT_TYPE_NAME (node));
 
 	return NULL;
 }
@@ -165,6 +152,9 @@ gpa_node_verify (GPANode *node)
 
 	if (GPA_NODE_GET_CLASS (node)->verify)
 		return GPA_NODE_GET_CLASS (node)->verify (node);
+	else
+		g_error ("Can't verify the \"%s\" node because the \"%s\" Class does not have a "
+			 "verify method.", gpa_node_id (node), G_OBJECT_TYPE_NAME (node));
 
 	return TRUE;
 }
@@ -177,41 +167,23 @@ gpa_node_get_value (GPANode *node)
 
 	if (GPA_NODE_GET_CLASS (node)->get_value)
 		return GPA_NODE_GET_CLASS (node)->get_value (node);
+	else
+		g_warning ("Can't get_valued from \"%s\" node because the \"%s\" Class does not have a "
+			   "get_value method.", gpa_node_id (node), G_OBJECT_TYPE_NAME (node));
 
 	return NULL;
 }
 
-gboolean
-gpa_node_set_value (GPANode *node, const guchar *value)
-{
-	gboolean ret;
-
-	g_return_val_if_fail (node != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_NODE (node), FALSE);
-
-	ret = FALSE;
-
-	if (GPA_NODE_GET_CLASS (node)->set_value)
-		ret = GPA_NODE_GET_CLASS (node)->set_value (node, value);
-
-	if (ret) {
-		gpa_node_request_modified (node, GPA_NODE_MODIFIED_FLAG);
-	}
-
-	return ret;
-}
-
-/* NB! ref->parent == node is not invariant due to references */
-
 GPANode *
-gpa_node_get_child (GPANode *node, GPANode *ref)
+gpa_node_get_child (GPANode *node, GPANode *previous_child)
 {
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
-	g_return_val_if_fail (!ref || GPA_IS_NODE (ref), NULL);
+	g_return_val_if_fail (!previous_child || GPA_IS_NODE (previous_child), NULL);
 
 	if (GPA_NODE_GET_CLASS (node)->get_child)
-		return GPA_NODE_GET_CLASS (node)->get_child (node, ref);
+		return GPA_NODE_GET_CLASS (node)->get_child (node, previous_child);
+	/* Not all nodes have childs (GPAValue for example), method is optional */
 
 	return NULL;
 }
@@ -219,45 +191,51 @@ gpa_node_get_child (GPANode *node, GPANode *ref)
 GPANode *
 gpa_node_lookup (GPANode *node, const guchar *path)
 {
+	GPANode *ret = NULL;
+	
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 	g_return_val_if_fail (!*path || isalnum (*path), NULL);
 
+	/* FIMXE: Why do we allow lookup without a path? (Chema) */
 	if (!*path) {
 		gpa_node_ref (node);
 		return node;
 	}
-
+	
 	if (GPA_NODE_GET_CLASS (node)->lookup)
-		return GPA_NODE_GET_CLASS (node)->lookup (node, path);
+		ret = GPA_NODE_GET_CLASS (node)->lookup (node, path);
+	else
+		g_warning ("Can't do a node_lookup of \"%s\" with the \"%s\" path node because the \"%s\" Class does not "
+			   "have a _lookup method.", gpa_node_id (node), path, G_OBJECT_TYPE_NAME (node));
 
-	return NULL;
+	return ret;
+}
+
+gboolean
+gpa_node_set_value (GPANode *node, const guchar *value)
+{
+	gboolean ret = FALSE;
+
+	g_return_val_if_fail (node != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_NODE (node), FALSE);
+	g_return_val_if_fail (value != NULL, FALSE);
+
+	if (GPA_NODE_GET_CLASS (node)->set_value) {
+		ret = GPA_NODE_GET_CLASS (node)->set_value (node, value);
+		gpa_enter ("Request modified from gpa_node_set_value Node:[%d]", GPOINTER_TO_INT (node));
+		gpa_node_request_modified (node, GPA_NODE_MODIFIED_FLAG);
+		gpa_leave ("Request modified from gpa_node_set_value");
+	} else {
+		g_warning ("Can't set_valued of \"%s\" to \"%s\" because the \"%s\" Class does not have a "
+			   "set_value method.", gpa_node_id (node), value, G_OBJECT_TYPE_NAME (node));
+	}
+
+	return ret;
 }
 
 /* Signal stuff */
-
-void
-gpa_node_request_modified (GPANode *node, guint flags)
-{
-	g_return_if_fail (node != NULL);
-	g_return_if_fail (GPA_IS_NODE (node));
-
-	if (!(GPA_NODE_FLAGS (node) & GPA_NODE_MODIFIED_FLAG)) {
-		GPA_NODE_SET_FLAGS (node, GPA_NODE_MODIFIED_FLAG);
-		if (node->parent) {
-			gpa_node_request_modified (node->parent, flags);
-		} else {
-			guint id;
-			id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (node), "idle_id"));
-			if (id == 0) {
-				id = g_idle_add ((GSourceFunc) gpa_node_modified_idle_hook, node);
-				g_object_set_data (G_OBJECT (node), "idle_id", GUINT_TO_POINTER (id));
-			}
-		}
-	}
-}
-
 void
 gpa_node_emit_modified (GPANode *node, guint flags)
 {
@@ -265,7 +243,7 @@ gpa_node_emit_modified (GPANode *node, guint flags)
 
 	GPA_NODE_UNSET_FLAGS (node, GPA_NODE_MODIFIED_FLAG);
 
-	g_signal_emit (G_OBJECT (node), node_signals[MODIFIED], 0, flags);
+	g_signal_emit (G_OBJECT (node), node_signals [MODIFIED], 0, flags);
 
 	gpa_node_unref (node);
 }
@@ -280,6 +258,40 @@ gpa_node_modified_idle_hook (GPANode *node)
 	return FALSE;
 }
 
+void
+gpa_node_request_modified (GPANode *node, guint flags)
+{
+	guint id;
+
+	g_return_if_fail (node != NULL);
+	g_return_if_fail (GPA_IS_NODE (node));
+
+	gpa_enter ("gpa_node_request_modified [%s][%d]", gpa_node_id (node), GPOINTER_TO_INT (node));
+
+	if (GPA_NODE_FLAGS (node) & GPA_NODE_MODIFIED_FLAG) {
+		gpa_leave ("gpa_node_request_modified (Allready marked) [%s]", gpa_node_id (node));
+		return;
+	}
+	
+	GPA_NODE_SET_FLAGS (node, GPA_NODE_MODIFIED_FLAG);
+
+	if (node->parent) {
+		gpa_node_request_modified (node->parent, flags);
+		/* FIXME: Lauris had a return here, so that the parent will only get the request
+		 * and not the child. Figure out if removing this return is ok (Chema)
+		 */
+	}
+	
+	id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (node), "idle_id"));
+
+	if (id == 0) {
+		id = g_idle_add ((GSourceFunc) gpa_node_modified_idle_hook, node);
+		g_object_set_data (G_OBJECT (node), "idle_id", GUINT_TO_POINTER (id));
+	} 
+
+	gpa_leave ("gpa_node_request_modified (3) [%s]", gpa_node_id (node));
+}
+
 /* Public methods */
 
 GPANode *
@@ -299,39 +311,48 @@ gpa_node_unref (GPANode *node)
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 
-	g_object_ref (G_OBJECT (node));
+	g_object_unref (G_OBJECT (node));
 
 	return NULL;
 }
 
-/* fixme: remove this or make const (Lauris) */
-
-guchar *
+const guchar *
 gpa_node_id (GPANode *node)
 {
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 
-	if (node->qid) return g_strdup (gpa_quark_to_string (node->qid));
+	if (node->qid)
+		return gpa_quark_to_string (node->qid);
 
 	return NULL;
 }
 
 /* These return referenced node or NULL */
-
 GPANode *
 gpa_node_get_parent (GPANode *node)
 {
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 
-	if (node->parent) gpa_node_ref (node->parent);
+	if (node->parent)
+		gpa_node_ref (node->parent);
 
 	return node->parent;
 }
 
+/**
+ * gpa_node_get_child_from_path:
+ * @node: 
+ * @path: 
+ * 
+ * This is just the public version of gpa_node_lookup. It gets you a child inside a tree
+ * given by the @path
+ * 
+ * Return Value: the GPANode * below the @node leaf after having walked @path
+ **/
 GPANode *
-gpa_node_get_path_node (GPANode *node, const guchar *path)
+gpa_node_get_child_from_path (GPANode *node, const guchar *path)
 {
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
@@ -346,41 +367,53 @@ gpa_node_get_path_node (GPANode *node, const guchar *path)
 guchar *
 gpa_node_get_path_value (GPANode *node, const guchar *path)
 {
-	GPANode *ref;
+	GPANode *child;
 
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 	g_return_val_if_fail (!*path || isalnum (*path), NULL);
 
-	ref = gpa_node_lookup (node, path);
+	child = gpa_node_lookup (node, path);
 
-	if (ref) {
+	if (child) {
 		guchar *value;
-		value = gpa_node_get_value (ref);
-		gpa_node_unref (ref);
+		value = gpa_node_get_value (child);
+		gpa_node_unref (child);
 		return value;
 	}
 
 	return NULL;
 }
 
+/**
+ * gpa_node_set_path_value:
+ * @node: 
+ * @path: 
+ * @value: 
+ * 
+ * It might return TRUE even when the value can't be set, for example
+ * if you are setting List.DefaultChild  and DefaultChild doesn't exist we can't
+ * know offhand if is valid because it is done with a signal emission
+ * 
+ * Return Value: 
+ **/
 gboolean
-gpa_node_set_path_value (GPANode *node, const guchar *path, const guchar *value)
+gpa_node_set_path_value (GPANode *parent, const guchar *path, const guchar *value)
 {
-	GPANode *ref;
+	GPANode *node;
 
-	g_return_val_if_fail (node != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_NODE (node), FALSE);
+	g_return_val_if_fail (parent != NULL, FALSE);
+	g_return_val_if_fail (GPA_IS_NODE (parent), FALSE);
 	g_return_val_if_fail (path != NULL, FALSE);
 	g_return_val_if_fail (!*path || isalnum (*path), FALSE);
 
-	ref = gpa_node_lookup (node, path);
+	node = gpa_node_lookup (parent, path);
 
-	if (ref) {
+	if (node) {
 		gboolean ret;
-		ret = gpa_node_set_value (ref, value);
-		gpa_node_unref (ref);
+		ret = gpa_node_set_value (node, value);
+		gpa_node_unref (node);
 		return ret;
 	}
 
@@ -469,6 +502,7 @@ gpa_node_get_double_path_value (GPANode *node, const guchar *path, gdouble *valu
 gboolean
 gpa_node_get_length_path_value (GPANode *node, const guchar *path, gdouble *value)
 {
+	gchar *loc, *e;
 	guchar *v;
 	
 	g_return_val_if_fail (node != NULL, FALSE);
@@ -479,26 +513,26 @@ gpa_node_get_length_path_value (GPANode *node, const guchar *path, gdouble *valu
 
 	v = gpa_node_get_path_value (node, path);
 
-	if (v != NULL) {
-		gchar *loc, *e;
-		loc = setlocale (LC_NUMERIC, NULL);
-		setlocale (LC_NUMERIC, "C");
-		*value = strtod (v, &e);
-		if (e) {
-			if (!strcmp (e, "mm")) {
-				*value *= MM2PT;
-			} else if (!strcmp (e, "cm")) {
-				*value *= CM2PT;
-			} else if (!strcmp (e, "in")) {
-				*value *= IN2PT;
-			}
-		}
-		g_free (v);
-		setlocale (LC_NUMERIC, loc);
-		return TRUE;
-	}
+	if (v == NULL)
+		return FALSE;
 	
-	return FALSE;
+	loc = setlocale (LC_NUMERIC, NULL);
+	setlocale (LC_NUMERIC, "C");
+	*value = strtod (v, &e);
+	if (e) {
+		if (!strcmp (e, "mm")) {
+			*value *= MM2PT;
+		} else if (!strcmp (e, "cm")) {
+			*value *= CM2PT;
+		} else if (!strcmp (e, "in")) {
+			*value *= IN2PT;
+		}
+	}
+
+	g_free (v);
+	setlocale (LC_NUMERIC, loc);
+	
+	return TRUE;
 }
 
 gboolean
@@ -549,12 +583,9 @@ gpa_node_set_double_path_value (GPANode *node, const guchar *path, gdouble value
 GPANode *
 gpa_defaults (void)
 {
-	GPANode *config;
-
+	static GPANode *config = NULL;
+	
 	config = gpa_config_new ();
 
 	return config;
 }
-
-
-
